@@ -28,7 +28,7 @@ from fluids.constants import R
 from thermo.activity import GibbsExcess
 from thermo import *
 import numpy as np
-from fluids.numerics import jacobian, hessian, derivative, normalize, assert_close, assert_close1d, assert_close2d, assert_close3d
+from fluids.numerics import jacobian, hessian, derivative, normalize, assert_close, assert_close1d, assert_close2d, assert_close3d, linspace
 from thermo.test_utils import check_np_output_activity
 import pickle
 
@@ -76,7 +76,14 @@ def test_DDBST_example():
     cs_ddbst = [[0.0, 7.91073e-3, 8.68371e-4], [-7.47788e-3, 0.0, 3.1e-5], [-1.24796e-3, 3e-5, 0.0]]
 
     dis = eis = fis = [[0.0]*N for _ in range(N)]
+    
+    # Do a base test with no parameters just to check it works
+    params = Wilson.from_DDBST_as_matrix(Vs=Vs_ddbst, unit_conversion=False)
+    # Do a base test with optional parameters missing
+    params = Wilson.from_DDBST_as_matrix(Vs=Vs_ddbst, ais=as_ddbst, bis=bs_ddbst,
+                                cis=cs_ddbst, unit_conversion=False)
 
+    # Do the test
     params = Wilson.from_DDBST_as_matrix(Vs=Vs_ddbst, ais=as_ddbst, bis=bs_ddbst,
                                 cis=cs_ddbst, dis=dis, eis=eis, fis=fis, unit_conversion=False)
 
@@ -410,6 +417,60 @@ def test_DDBST_example():
     GE2 = Wilson.from_json(GE.as_json())
     assert GE2.__dict__ == GE.__dict__
 
+def test_wilson_parameter_different_inputs_errors():
+    T = 353.15
+    N = 2
+    xs = [0.25, 0.75]
+    
+    lamdasA = [[0.0, -1.4485128161164418], [1.448512816116442, 0.0]]
+    lambdasB = lambdasC = lambdasD = lambdasE = lambdasF =  [[0.0, 0],[0, 0.0]]
+
+    gammas_expect = [0.6104842750833774, 0.8634230875164046]
+    ABCDEF = (lamdasA, lambdasB, lambdasC, lambdasD, lambdasE, lambdasF)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA, lambdasB, lambdasC, lambdasD, lambdasE)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA, lambdasB, lambdasC, lambdasD)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA, lambdasB, lambdasC)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA, lambdasB)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA,)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    ABCDEF = (lamdasA, None, None, None, None, None)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), gammas_expect, rtol=1e-13)
+    
+    with pytest.raises(ValueError):
+         Wilson(T=T, xs=xs, ABCDEF=(lamdasA, None, None, []))
+    with pytest.raises(ValueError):
+         Wilson(T=T, xs=xs, ABCDEF=(lamdasA, None, None, [1,]))
+    with pytest.raises(ValueError):
+         Wilson(T=T, xs=xs, ABCDEF=(lamdasA, None, None, [1, 1]))
+    with pytest.raises(ValueError):
+         Wilson(T=T, xs=xs, ABCDEF=(lamdasA, None, None, [[0.0, 9.64e-8], [1.53e-7, 0.0, 1.11e-7], [7.9e-8, 2.276e-8, 0]]))
+    with pytest.raises(ValueError):
+         Wilson(T=T, xs=xs, ABCDEF=(lamdasA, None, None,  [[0.0, 9.64e-8, 8.94e-8], [1.53e-7, 0.0, 1.11e-7]]))
+    
+    assert_close1d(Wilson(T=T, xs=xs).gammas(), [1, 1], rtol=1e-13)
+
+    # No coefficients
+    ABCDEF = (None, None, None, None, None, None)
+    obj = Wilson(T=T, xs=xs, ABCDEF=ABCDEF)
+    assert_close1d(obj.gammas(), [1, 1], rtol=1e-13)
 def test_multicomnent_madeup():
     T=273.15+70
     xs = [1/7.0]*7
@@ -724,8 +785,47 @@ def test_Wilson_numpy_output_correct_array_internal_ownership():
     lambda_coeffs = [[[A[i][j], B[i][j], C[i][j], D[i][j], E[i][j], F[i][j]] for j in range(N)] for i in range(N)]
     xs = [0.229, 0.175, 0.596]
     modelnp = Wilson(T=T, xs=np.array(xs), lambda_coeffs=np.array(lambda_coeffs))
-    for name in ('lambda_coeffs_A', 'lambda_coeffs_B', 'lambda_coeffs_C',
-                 'lambda_coeffs_D', 'lambda_coeffs_E', 'lambda_coeffs_F'):
+    for name in ('lambda_as', 'lambda_bs', 'lambda_cs',
+                 'lambda_ds', 'lambda_es', 'lambda_fs'):
         obj = getattr(modelnp, name)
         assert obj.flags.c_contiguous
         assert obj.flags.owndata
+
+
+def test_Wilson_one_component():
+    GE = Wilson(T=300, xs=[1], ABCDEF=([[0.0]], [[0.0]], [[0.0]], [[0.0]], [[0.0]], [[0.0]]))
+    for s in GE._point_properties:
+        if hasattr(GE, s):
+            res = getattr(GE, s)()
+
+
+def test_Wilson_chemsep():
+    from thermo.interaction_parameters import IPDB
+    CAS1 = '64-17-5'
+    CAS2 = '7732-18-5'
+    N = 2
+    T = 273.15 + 70
+    tausC = tausD = tausE = tausF = [[0.0]*N for _ in range(N)]
+    tausB = IPDB.get_ip_asymmetric_matrix(name='ChemSep Wilson', CASs=[CAS1, CAS2], ip='bij')
+    tausA = IPDB.get_ip_asymmetric_matrix(name='ChemSep Wilson', CASs=[CAS1, CAS2], ip='aij')
+    ABCDEF = (tausA, tausB, tausC, tausD, tausE, tausF)
+    GE = Wilson(T=T, xs=[0.252, 1-0.252], ABCDEF=ABCDEF)
+    gammas = GE.gammas()
+    # Checked against ChemSep TPxy plot/data table - shows 5 decimal places
+    assert_close1d(gammas, [1.9573311040154513, 1.1600677182620136], rtol=1e-12)
+
+def test_wilson_gammas_binaries():
+    GE = Wilson(T=343.15, xs=[0.252, 0.748], lambda_as=[[0.0, -1.1769274893976625], [1.1769274893976625, 0.0]], lambda_bs=[[0.0, -192.38082765657816], [-480.8011032813958, 0.0]])
+    lambdas = GE.lambdas()
+    lambda12 = lambdas[0][1]
+    lambda21 = lambdas[1][0]
+    
+    xs = []
+    gammas_object = []
+    for x in linspace(0, 1, 5):
+        xs.append(x)
+        xs.append(1-x)
+        gammas_object.extend(GE.to_T_xs(T=GE.T, xs=xs[-2:]).gammas())
+    
+    all_gammas = wilson_gammas_binaries(xs, lambda12, lambda21)
+    assert_close1d(all_gammas, gammas_object, rtol=1e-13)
