@@ -140,12 +140,6 @@ class SurfaceTension(TDependentProperty):
         None to not extrapolate; see
         :obj:`TDependentProperty <thermo.utils.TDependentProperty>`
         for a full list of all options, [-]
-    poly_fit : tuple(float, float, list[float]), optional
-        Tuple of (Tmin, Tmax, coeffs) representing a prefered fit to the
-        surface tension of the compound; the coefficients are evaluated with
-        horner's method, and the input variable and output are transformed by
-        the default transformations of this object; used instead of any other
-        default low-pressure method if provided. [-]
     method : str or None, optional
         If specified, use this method by default and do not use the ranked
         sorting; an exception is raised if this is not a valid method for the
@@ -263,7 +257,7 @@ class SurfaceTension(TDependentProperty):
 
     def __init__(self, MW=None, Tb=None, Tc=None, Pc=None, Vc=None, Zc=None,
                  omega=None, StielPolar=None, Hvap_Tb=None, CASRN='', Vml=None,
-                 Cpl=None, extrapolation=None, **kwargs):
+                 Cpl=None, extrapolation='EQ106_AB', **kwargs):
         self.MW = MW
         self.Tb = Tb
         self.Tc = Tc
@@ -419,7 +413,13 @@ class SurfaceTension(TDependentProperty):
         elif method == ALEEM:
             Cpl = self.Cpl(T) if hasattr(self.Cpl, '__call__') else self.Cpl
             Cpl = property_molar_to_mass(Cpl, self.MW)
-            Vml = self.Vml(T) if hasattr(self.Vml, '__call__') else self.Vml
+            try:
+                Vml = self.Vml.T_dependent_property(T)
+            except:
+                try:
+                    Vml = self.Vml(T)
+                except:
+                    Vml = self.Vml
             rhol = Vm_to_rho(Vml, self.MW)
             sigma = Aleem(T=T, MW=self.MW, Tb=self.Tb, rhol=rhol, Hvap_Tb=self.Hvap_Tb, Cpl=Cpl)
         else:
