@@ -180,11 +180,14 @@ def transform_complete_thermo(replaced, __funcs, __all__, normal, vec=False):
                  'regular_solution.regular_solution_d3GE_dxixjxks',
                  'regular_solution.regular_solution_gammas',
                  'regular_solution.RegularSolution',
+                 'regular_solution.regular_solution_gammas_binaries',
+                 'regular_solution.regular_solution_gammas_binaries_jac',
 
                  'wilson.Wilson',
                  'wilson.wilson_xj_Lambda_ijs', 'wilson.wilson_d2GE_dTdxs',
                  'wilson.wilson_dGE_dxs', 'wilson.wilson_d2GE_dxixjs',
                  'wilson.wilson_d3GE_dxixjxks', 'wilson.wilson_gammas',
+                 'wilson.wilson_gammas_binaries', 'wilson.wilson_gammas_binaries_jac',
 
                  'uniquac.UNIQUAC',
                  'uniquac.uniquac_phis',
@@ -194,6 +197,7 @@ def transform_complete_thermo(replaced, __funcs, __all__, normal, vec=False):
                  'uniquac.uniquac_d2phis_dxixjs',
                  'uniquac.uniquac_dGE_dxs',
                  'uniquac.uniquac_d2GE_dTdxs',
+                 'uniquac.UNIQUAC_gammas_binaries',
 
                  'nrtl.NRTL',
                  'nrtl.nrtl_gammas',
@@ -212,6 +216,8 @@ def transform_complete_thermo(replaced, __funcs, __all__, normal, vec=False):
                  'nrtl.nrtl_dGE_dxs',
                  'nrtl.nrtl_d2GE_dxixjs',
                  'nrtl.nrtl_d2GE_dTdxs',
+                 'nrtl.NRTL_gammas_binaries',
+                 'nrtl.NRTL_gammas_binaries_jac',
 
                 'unifac.unifac_psis',
                 'unifac.unifac_dpsis_dT',
