@@ -42,21 +42,24 @@ Wilson Functional Calculations
 ==============================
 .. autofunction:: Wilson_gammas
 
+Wilson Regression Calculations
+==============================
+.. autofunction:: wilson_gammas_binaries
 
 '''
 
 from __future__ import division
 from math import log, exp
 from fluids.constants import R
-from fluids.numerics import numpy as np
+from fluids.numerics import numpy as np, trunc_exp
 from thermo.activity import GibbsExcess, interaction_exp, dinteraction_exp_dT, d2interaction_exp_dT2, d3interaction_exp_dT3
 
 try:
-    array, zeros, npsum, nplog = np.array, np.zeros, np.sum, np.log
+    array, zeros, npsum, nplog, ones = np.array, np.zeros, np.sum, np.log, np.ones
 except (ImportError, AttributeError):
     pass
 
-__all__ = ['Wilson', 'Wilson_gammas']
+__all__ = ['Wilson', 'Wilson_gammas', 'wilson_gammas_binaries', 'wilson_gammas_binaries_jac']
 
 
 def wilson_xj_Lambda_ijs(xs, lambdas, N, xj_Lambda_ijs=None):
@@ -206,11 +209,169 @@ def wilson_gammas(xs, N, lambdas, xj_Lambda_ijs_inv, gammas=None, vec0=None):
 
     return gammas
 
+MIN_LAMBDA_WILSON = 1e-20
+
+def wilson_gammas_binaries(xs, lambda12, lambda21, calc=None):
+    r'''Calculates activity coefficients at fixed `lambda` values for
+    a binary system at a series of mole fractions. This is used for 
+    regression of `lambda` parameters. This function is highly optimized,
+    and operates on multiple points at a time.
+    
+    .. math::
+        \ln \gamma_1 = -\ln(x_1 + \Lambda_{12}x_2) + x_2\left(
+        \frac{\Lambda_{12}}{x_1 + \Lambda_{12}x_2} 
+        - \frac{\Lambda_{21}}{x_2 + \Lambda_{21}x_1}
+        \right)
+
+    .. math::
+        \ln \gamma_2 = -\ln(x_2 + \Lambda_{21}x_1) - x_1\left(
+        \frac{\Lambda_{12}}{x_1 + \Lambda_{12}x_2} 
+        - \frac{\Lambda_{21}}{x_2 + \Lambda_{21}x_1}
+        \right)
+
+    Parameters
+    ----------
+    xs : list[float]
+        Liquid mole fractions of each species in the format
+        x0_0, x1_0, (component 1 point1, component 2 point 1),
+        x0_1, x1_1, (component 1 point2, component 2 point 2), ...
+        [-]
+    lambda12 : float
+        `lambda` parameter for 12, [-]
+    lambda21 : float
+        `lambda` parameter for 21, [-]
+    gammas : list[float], optional
+        Array to store the activity coefficient for each species in the liquid 
+        mixture, indexed the same as `xs`; can be omitted or provided
+        for slightly better performance [-]
+
+    Returns
+    -------
+    gammas : list[float]
+        Activity coefficient for each species in the liquid mixture,
+        indexed the same as `xs`, [-]
+
+    Notes
+    -----
+    The lambda values are hard-coded to replace values under zero which are
+    mathematically impossible, with a very small number. This is helpful for
+    regression which might try to make those values negative.
+    
+    Examples
+    --------
+    >>> wilson_gammas_binaries([.1, .9, 0.3, 0.7, .85, .15], 0.1759, 0.7991)
+    [3.42989, 1.03432, 1.74338, 1.21234, 1.01766, 2.30656]
+    '''
+    if lambda12 < MIN_LAMBDA_WILSON:
+        lambda12 = MIN_LAMBDA_WILSON
+    if lambda21 < MIN_LAMBDA_WILSON:
+        lambda21 = MIN_LAMBDA_WILSON
+    pts = len(xs)//2 # Always even
+    
+    if calc is None:
+        allocate_size = (pts*2)
+        calc = [0.0]*allocate_size
+
+    for i in range(pts):
+        i2 = i*2
+        x1 = xs[i2]
+        x2 = 1.0 - x1
+        
+        c0 = 1.0/(x1 + x2*lambda12)
+        c1 = 1.0/(x2 + x1*lambda21)
+        c3 = lambda12*c0 - lambda21*c1
+        
+        calc[i2] = trunc_exp(c3*x2)*c0
+        calc[i2 + 1] = trunc_exp(-c3*x1)*c1
+    return calc
+
+    
+'''
+Actually readable expression of `wilson_gammas_binaries`:
+    
+    if lambda12 < MIN_LAMBDA_WILSON:
+        lambda12 = MIN_LAMBDA_WILSON
+    if lambda21 < MIN_LAMBDA_WILSON:
+        lambda21 = MIN_LAMBDA_WILSON
+    pts = int(len(xs)/2) # Always even
+    allocate_size = (pts*2)
+    
+#    lambdas = ones((2,2)) # numba: uncomment
+    lambdas = [[1.0, 1.0], [1.0, 1.0]] # numba: delete
+    lambdas[0][1] = lambda12
+    lambdas[1][0] = lambda21
+    
+    if calc is None:
+        calc = [0.0]*allocate_size
+
+    xj_Lambda_ijs_vec = [0.0]*2
+    vec0 = [0.0]*2
+    gammas = [0.0]*2
+    xs_pt = [0.0]*2
+
+    for i in range(pts):
+        i2 = i*2
+        xs_pt[0] = xs[i2]
+        xs_pt[1] = 1.0 - xs_pt[0]
+        
+        xj_Lambda_ijs = wilson_xj_Lambda_ijs(xs_pt, lambdas, N=2, xj_Lambda_ijs=xj_Lambda_ijs_vec)
+        xj_Lambda_ijs[0] = 1.0/xj_Lambda_ijs[0]
+        xj_Lambda_ijs[1] = 1.0/xj_Lambda_ijs[1]
+        gammas = wilson_gammas(xs_pt, N=2, lambdas=lambdas, xj_Lambda_ijs_inv=xj_Lambda_ijs, gammas=gammas, vec0=vec0)
+        calc[i2] = gammas[0]
+        calc[i2 + 1] = gammas[1]
+    return calc
+'''
+    
+def wilson_gammas_binaries_jac(xs, lambda12, lambda21, calc=None):
+    if lambda12 < MIN_LAMBDA_WILSON:
+        lambda12 = MIN_LAMBDA_WILSON
+    if lambda21 < MIN_LAMBDA_WILSON:
+        lambda21 = MIN_LAMBDA_WILSON
+    pts = len(xs)//2 # Always even
+    
+    if calc is None:
+        allocate_size = (pts*2)
+        calc = np.zeros((allocate_size, 2))
+    
+    for i in range(pts):
+        i2 = i*2
+        x1 = xs[i2]
+        x2 = 1.0 - x1
+        
+        c0 = lambda12*x2
+        c1 = c0 + x1
+        c2 = 1.0/c1
+        c3 = lambda21*x1
+        c4 = c3 + x2
+        c5 = 1.0/c4
+        c6 = c2*lambda12 - c5*lambda21
+        c7 = trunc_exp(c6*x2)
+        c8 = c2*c5
+        c9 = trunc_exp(-c6*x1)
+
+        calc[i2][0] = -c7*lambda12*x2*x2*c2*c2*c2
+        calc[i2][1] = c7*c8*x2*(c3*c5 - 1.0)
+        calc[i2 + 1][0] = c8*c9*x1*(c0*c2 - 1.0)
+        calc[i2 + 1][1] = -c9*lambda21*x1*x1*c5*c5*c5
+    return calc
+
+
 class Wilson(GibbsExcess):
     r'''Class for representing an a liquid with excess gibbs energy represented
     by the Wilson equation. This model is capable of representing most
     nonideal liquids for vapor-liquid equilibria, but is not recommended for
     liquid-liquid equilibria.
+    
+    The two basic equations are as follows; all other properties are derived 
+    from these.
+
+    .. math::
+        g^E = -RT\sum_i x_i \ln\left(\sum_j x_j \lambda_{i,j} \right)
+
+    .. math::
+        \Lambda_{ij} = \exp\left[a_{ij}+\frac{b_{ij}}{T}+c_{ij}\ln T
+                + d_{ij}T + \frac{e_{ij}}{T^2} + f_{ij}{T^2}\right]
 
     Parameters
     ----------
@@ -220,24 +381,22 @@ class Wilson(GibbsExcess):
         Mole fractions, [-]
     lambda_coeffs : list[list[list[float]]], optional
         Wilson parameters, indexed by [i][j] and then each value is a 6
-        element list with parameters [`a`, `b`, `c`, `d`, `e`, `f`];
-        either `lambda_coeffs` or `ABCDEF` are required, [-]
-    ABCDEF : tuple[list[list[float]], 6], optional
-        Contains the following. One of `lambda_coeffs` or `ABCDEF` are
-        required, [-]
-
-        a : list[list[float]]
-            `a` parameters used in calculating :obj:`Wilson.lambdas`, [-]
-        b : list[list[float]]
-            `b` parameters used in calculating :obj:`Wilson.lambdas`, [K]
-        c : list[list[float]]
-            `c` parameters used in calculating :obj:`Wilson.lambdas`, [-]
-        d : list[list[float]]
-            `d` paraemeters used in calculating :obj:`Wilson.lambdas`, [1/K]
-        e : list[list[float]]
-            `e` parameters used in calculating :obj:`Wilson.lambdas`, [K^2]
-        f : list[list[float]]
-            `f` parameters used in calculating :obj:`Wilson.lambdas`, [1/K^2]
+        element list with parameters (`a`, `b`, `c`, `d`, `e`, `f`);
+        either `lambda_coeffs` or the lambda parameters are required, [various]
+    ABCDEF : tuple(list[list[float]], 6), optional
+        The lamba parameters can be provided as a tuple, [various]
+    lambda_as : list[list[float]], optional
+        `a` parameters used in calculating :obj:`Wilson.lambdas`, [-]
+    lambda_bs : list[list[float]], optional
+        `b` parameters used in calculating :obj:`Wilson.lambdas`, [K]
+    lambda_cs : list[list[float]], optional
+        `c` parameters used in calculating :obj:`Wilson.lambdas`, [-]
+    lambda_ds : list[list[float]], optional
+        `d` paraemeters used in calculating :obj:`Wilson.lambdas`, [1/K]
+    lambda_es : list[list[float]], optional
+        `e` parameters used in calculating :obj:`Wilson.lambdas`, [K^2]
+    lambda_fs : list[list[float]], optional
+        `f` parameters used in calculating :obj:`Wilson.lambdas`, [1/K^2]
 
     Attributes
     ----------
@@ -245,14 +404,104 @@ class Wilson(GibbsExcess):
         Temperature, [K]
     xs : list[float]
         Mole fractions, [-]
+    model_id : int
+        Unique identifier for the Wilson activity model, [-]
 
     Notes
     -----
     In addition to the methods presented here, the methods of its base class
     :obj:`thermo.activity.GibbsExcess` are available as well.
 
+    .. warning::
+        If parameters are ommited for all interactions, this model
+        reverts to :obj:`thermo.activity.IdealSolution`. In large systems it
+        is common to only regress parameters for the most important components;
+        set `lambda` parameters for other components to 0 to "ignore" them and
+        treat them as ideal components.
+    
+    This class works with python lists, numpy arrays, and can be accelerated
+    with Numba or PyPy quite effectively.
+
     Examples
     --------
+    **Example 1**
+    
+    This object-oriented class provides access to many more thermodynamic
+    properties than :obj:`Wilson_gammas`, but it can also be used like that 
+    function. In the following example, `gammas` are calculated with both
+    functions. The `lambdas` cannot be specified in this class; but fixed
+    values can be converted with the `log` function so that fixed values will
+    be obtained.
+    
+    >>> Wilson_gammas([0.252, 0.748], [[1, 0.154], [0.888, 1]])
+    [1.881492608717, 1.165577493112]
+    >>> GE = Wilson(T=300.0, xs=[0.252, 0.748], lambda_as=[[0, log(0.154)], [log(0.888), 0]])
+    >>> GE.gammas()
+    [1.881492608717, 1.165577493112]
+    
+    We can check that the same lambda values were computed as well, and that
+    there is no temperature dependency:
+        
+    >>> GE.lambdas()
+    [[1.0, 0.154], [0.888, 1.0]]
+    >>> GE.dlambdas_dT()
+    [[0.0, 0.0], [0.0, 0.0]]
+    
+    In this case, there is no temperature dependency in the Wilson model as the
+    `lambda` values are fixed, so the excess enthalpy is always zero. Other
+    properties are not always zero.
+    
+    >>> GE.HE(), GE.CpE()
+    (0.0, 0.0)
+    >>> GE.GE(), GE.SE(), GE.dGE_dT()
+    (683.165839398, -2.277219464, 2.2772194646)
+    
+    **Example 2**
+    
+    ChemSep is a (partially) free program for modeling distillation. Besides
+    being a wonderful program, it also ships with a permissive license several
+    sets of binary interaction parameters. The Wilson parameters in it can
+    be accessed from Thermo as follows. In the following case, we compute
+    activity coefficients of the ethanol-water system at mole fractions of
+    [.252, 0.748].
+    
+    >>> from thermo.interaction_parameters import IPDB
+    >>> CAS1, CAS2 = '64-17-5', '7732-18-5'
+    >>> lambda_as = IPDB.get_ip_asymmetric_matrix(name='ChemSep Wilson', CASs=[CAS1, CAS2], ip='aij')
+    >>> lambda_bs = IPDB.get_ip_asymmetric_matrix(name='ChemSep Wilson', CASs=[CAS1, CAS2], ip='bij')
+    >>> GE = Wilson(T=273.15+70, xs=[.252, .748], lambda_as=lambda_as, lambda_bs=lambda_bs)
+    >>> GE.gammas()
+    [1.95733110, 1.1600677]
+
+    In ChemSep, the form of the Wilson `lambda` equation is
+    
+    .. math::
+        \Lambda_{ij} = \frac{V_j}{V_i}\exp\left( \frac{-A_{ij}}{RT}\right)
+    
+    The parameters were converted to the form used by Thermo as follows:
+        
+    .. math::
+        a_{ij} = \log\left(\frac{V_j}{V_i}\right)
+        
+    .. math::
+        b_{ij} = \frac{-A_{ij}}{R}= \frac{-A_{ij}}{ 1.9872042586408316}
+        
+    
+    This system was chosen because there is also a sample problem for the same
+    components from the DDBST which can be found here:
+    http://chemthermo.ddbst.com/Problems_Solutions/Mathcad_Files/P05.01a%20VLE%20Behavior%20of%20Ethanol%20-%20Water%20Using%20Wilson.xps
+    
+    In that example, with different data sets and parameters, they obtain at
+    the same conditions activity coefficients of [1.881, 1.165]. Different
+    sources of parameters for the same system will generally have similar 
+    behavior if regressed in the same temperature range. As higher order
+    `lambda` parameters are added, models become more likely to behave 
+    differently. It is recommended in [3]_ to regress the minimum number of
+    parameters required.
+    
+    
+    **Example 3**
+    
     The DDBST has published some sample problems which are fun to work with.
     Because the DDBST uses a different equation form for the coefficients than
     this model implements, we must initialize the :obj:`Wilson` object with
@@ -267,9 +516,9 @@ class Wilson(GibbsExcess):
     >>> dis = eis = fis = [[0.0]*N for _ in range(N)]
     >>> params = Wilson.from_DDBST_as_matrix(Vs=Vs_ddbst, ais=as_ddbst, bis=bs_ddbst, cis=cs_ddbst, dis=dis, eis=eis, fis=fis, unit_conversion=False)
     >>> xs = [0.229, 0.175, 0.596]
-    >>> GE = Wilson(T=T, xs=xs, ABCDEF=params)
+    >>> GE = Wilson(T=T, xs=xs, lambda_as=params[0], lambda_bs=params[1], lambda_cs=params[2], lambda_ds=params[3], lambda_es=params[4], lambda_fs=params[5])
     >>> GE
-    Wilson(T=331.42, xs=[0.229, 0.175, 0.596], ABCDEF=([[0.0, 3.870101271243586, 0.07939943395502425], [-6.491263271243587, 0.0, -3.276991837288562], [0.8542855660449756, 6.906801837288562, 0.0]], [[0.0, -375.2835, -31.1208], [1722.58, 0.0, 1140.79], [-747.217, -3596.17, -0.0]], [[-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0]], [[-0.0, -0.00791073, -0.000868371], [0.00747788, -0.0, -3.1e-05], [0.00124796, -3e-05, -0.0]], [[-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0]], [[-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0], [-0.0, -0.0, -0.0]]))
+    Wilson(T=331.42, xs=[0.229, 0.175, 0.596], lambda_as=[[0.0, 3.870101271243586, 0.07939943395502425], [-6.491263271243587, 0.0, -3.276991837288562], [0.8542855660449756, 6.906801837288562, 0.0]], lambda_bs=[[0.0, -375.2835, -31.1208], [1722.58, 0.0, 1140.79], [-747.217, -3596.17, -0.0]], lambda_ds=[[-0.0, -0.00791073, -0.000868371], [0.00747788, -0.0, -3.1e-05], [0.00124796, -3e-05, -0.0]])
     >>> GE.GE(), GE.dGE_dT(), GE.d2GE_dT2()
     (480.2639266306882, 4.355962766232997, -0.029130384525017247)
     >>> GE.HE(), GE.SE(), GE.dHE_dT(), GE.dSE_dT()
@@ -283,6 +532,8 @@ class Wilson(GibbsExcess):
     http://chemthermo.ddbst.com/Problems_Solutions/Mathcad_Files/05.09%20Compare%20Experimental%20VLE%20to%20Wilson%20Equation%20Results.xps
 
 
+    **Example 4**
+    
     A simple example is given in [1]_; other textbooks sample problems are
     normally in the same form as this - with only volumes and the `a` term
     specified. The system is 2-propanol/water at 353.15 K, and the mole
@@ -295,20 +546,26 @@ class Wilson(GibbsExcess):
     >>> bis = cis = dis = eis = fis = [[0.0]*N for _ in range(N)]
     >>> params = Wilson.from_DDBST_as_matrix(Vs=Vs, ais=ais, bis=bis, cis=cis, dis=dis, eis=eis, fis=fis, unit_conversion=True)
     >>> xs = [0.25, 0.75]
-    >>> GE = Wilson(T=T, xs=xs, ABCDEF=params)
+    >>> GE = Wilson(T=T, xs=xs, lambda_as=params[0], lambda_bs=params[1], lambda_cs=params[2], lambda_ds=params[3], lambda_es=params[4], lambda_fs=params[5])
     >>> GE.gammas()
     [2.124064516, 1.1903745834]
 
     The activity coefficients given in [1]_ are [2.1244, 1.1904]; matching (
-    with a slight error from their use of 1.987 as a gas constant).
+    with a slight deviation from their use of 1.987 as a gas constant).
 
     References
-    ----------x
+    ----------
     .. [1] Smith, H. C. Van Ness Joseph M. Introduction to Chemical Engineering
        Thermodynamics 4th Edition, Joseph M. Smith, H. C. Van
        Ness, 1987.
+    .. [2] Kooijman, Harry A., and Ross Taylor. The ChemSep Book. Books on 
+       Demand Norderstedt, Germany, 2000.
+    .. [3] Gmehling, Jürgen, Michael Kleiber, Bärbel Kolbe, and Jürgen Rarey.
+       Chemical Thermodynamics for Process Simulation. John Wiley & Sons, 2019.
     '''
+    
     model_id = 200
+    
     @staticmethod
     def from_DDBST(Vi, Vj, a, b, c, d=0.0, e=0.0, f=0.0, unit_conversion=True):
         r'''Converts parameters for the wilson equation in the DDBST to the
@@ -382,7 +639,8 @@ class Wilson(GibbsExcess):
         return (a, b, c, d, e, f)
 
     @staticmethod
-    def from_DDBST_as_matrix(Vs, ais, bis, cis, dis, eis, fis,
+    def from_DDBST_as_matrix(Vs, ais=None, bis=None, cis=None, dis=None, 
+                             eis=None, fis=None,
                              unit_conversion=True):
         r'''Converts parameters for the wilson equation in the DDBST to the
         basis used in this implementation. Matrix wrapper around
@@ -393,17 +651,17 @@ class Wilson(GibbsExcess):
         Vs : list[float]
             Molar volume of component; needs only to be in consistent units,
             [cm^3/mol]
-        a : list[list[float]]
+        ais : list[list[float]]
             `a` parameters in DDBST form, [K]
-        b : list[list[float]]
+        bis : list[list[float]]
             `b` parameters in DDBST form, [-]
-        c : list[list[float]]
+        cis : list[list[float]]
             `c` parameters in DDBST form, [1/K]
-        d : list[list[float]], optional
+        dis : list[list[float]], optional
             `d` parameters in DDBST form, [-]
-        e : list[list[float]], optional
+        eis : list[list[float]], optional
             `e` parameters in DDBST form, [1/K^2]
-        f : list[list[float]], optional
+        fis : list[list[float]], optional
             `f` parameters in DDBST form, [K^2]
         unit_conversion : bool
             If True, the input coefficients are in units of cal/K/mol, and a
@@ -425,7 +683,20 @@ class Wilson(GibbsExcess):
         f : list[list[float]]
             `f` parameters in :obj:`Wilson` form, [1/K^2]
         '''
-        cmps = range(len(Vs))
+        N = len(Vs)
+        cmps = range(N)
+        if ais is None:
+            ais = [[0.0]*N for _ in range(N)]
+        if bis is None:
+            bis = [[0.0]*N for _ in range(N)]
+        if cis is None:
+            cis = [[0.0]*N for _ in range(N)]
+        if dis is None:
+            dis = [[0.0]*N for _ in range(N)]
+        if eis is None:
+            eis = [[0.0]*N for _ in range(N)]
+        if fis is None:
+            fis = [[0.0]*N for _ in range(N)]
         a_mat, b_mat, c_mat, d_mat, e_mat, f_mat = [], [], [], [], [], []
         for i in cmps:
             a_row, b_row, c_row, d_row, e_row, f_row = [], [], [], [], [], []
@@ -448,42 +719,104 @@ class Wilson(GibbsExcess):
             f_mat.append(f_row)
         return (a_mat, b_mat, c_mat, d_mat, e_mat, f_mat)
 
-    def __init__(self, T, xs, lambda_coeffs=None, ABCDEF=None):
+    def __init__(self, T, xs, lambda_coeffs=None, ABCDEF=None, lambda_as=None, lambda_bs=None,
+                 lambda_cs=None, lambda_ds=None, lambda_es=None, lambda_fs=None):
         self.T = T
         self.xs = xs
         self.scalar = scalar = type(xs) is list
-        if ABCDEF is not None:
-            (self.lambda_coeffs_A, self.lambda_coeffs_B, self.lambda_coeffs_C,
-            self.lambda_coeffs_D, self.lambda_coeffs_E, self.lambda_coeffs_F) = ABCDEF
-            self.N = N = len(self.lambda_coeffs_A)
+        self.N = N = len(xs)
+        
+        if ABCDEF is None:
+            ABCDEF = (lambda_as, lambda_bs, lambda_cs, lambda_ds, lambda_es, lambda_fs)
+        if lambda_coeffs is not None:
+            pass
         else:
-            if lambda_coeffs is not None:
-                if scalar:
-                    self.lambda_coeffs_A = [[i[0] for i in l] for l in lambda_coeffs]
-                    self.lambda_coeffs_B = [[i[1] for i in l] for l in lambda_coeffs]
-                    self.lambda_coeffs_C = [[i[2] for i in l] for l in lambda_coeffs]
-                    self.lambda_coeffs_D = [[i[3] for i in l] for l in lambda_coeffs]
-                    self.lambda_coeffs_E = [[i[4] for i in l] for l in lambda_coeffs]
-                    self.lambda_coeffs_F = [[i[5] for i in l] for l in lambda_coeffs]
-                else:
-                    self.lambda_coeffs_A = array(lambda_coeffs[:,:,0], order='C', copy=True)
-                    self.lambda_coeffs_B = array(lambda_coeffs[:,:,1], order='C', copy=True)
-                    self.lambda_coeffs_C = array(lambda_coeffs[:,:,2], order='C', copy=True)
-                    self.lambda_coeffs_D = array(lambda_coeffs[:,:,3], order='C', copy=True)
-                    self.lambda_coeffs_E = array(lambda_coeffs[:,:,4], order='C', copy=True)
-                    self.lambda_coeffs_F = array(lambda_coeffs[:,:,5], order='C', copy=True)
+            try:
+                all_lengths = tuple(len(coeffs) for coeffs in ABCDEF if coeffs is not None)
+                if len(set(all_lengths)) > 1:
+                    raise ValueError("Coefficient arrays of different size found: %s" %(all_lengths,))
+                all_lengths_inner = tuple(len(coeffs[0]) for coeffs in ABCDEF if coeffs is not None)
+                if len(set(all_lengths_inner)) > 1:
+                    raise ValueError("Coefficient arrays of different size found: %s" %(all_lengths_inner,))
+            except:
+                raise ValueError("Coefficients not input correctly")
 
+        if scalar:
+            zero_coeffs = [[0.0]*N for _ in range(N)]
+        else:
+            zero_coeffs = zeros((N, N))
+
+        if lambda_coeffs is not None:
+            if scalar:
+                self.lambda_as = [[i[0] for i in l] for l in lambda_coeffs]
+                self.lambda_bs = [[i[1] for i in l] for l in lambda_coeffs]
+                self.lambda_cs = [[i[2] for i in l] for l in lambda_coeffs]
+                self.lambda_ds = [[i[3] for i in l] for l in lambda_coeffs]
+                self.lambda_es = [[i[4] for i in l] for l in lambda_coeffs]
+                self.lambda_fs = [[i[5] for i in l] for l in lambda_coeffs]                
             else:
-                raise ValueError("`lambda_coeffs` or `ABCDEF` is required required")
-            self.N = N = len(lambda_coeffs)
+                self.lambda_as = array(lambda_coeffs[:,:,0], order='C', copy=True)
+                self.lambda_bs = array(lambda_coeffs[:,:,1], order='C', copy=True)
+                self.lambda_cs = array(lambda_coeffs[:,:,2], order='C', copy=True)
+                self.lambda_ds = array(lambda_coeffs[:,:,3], order='C', copy=True)
+                self.lambda_es = array(lambda_coeffs[:,:,4], order='C', copy=True)
+                self.lambda_fs = array(lambda_coeffs[:,:,5], order='C', copy=True)
+        else:
+            len_ABCDEF = len(ABCDEF)
+            if len_ABCDEF == 0 or ABCDEF[0] is None:
+                self.lambda_as = zero_coeffs
+            else:
+                self.lambda_as = ABCDEF[0]
+            if len_ABCDEF < 2 or ABCDEF[1] is None:
+                self.lambda_bs = zero_coeffs
+            else:
+                self.lambda_bs = ABCDEF[1]
+            if len_ABCDEF < 3 or ABCDEF[2] is None:
+                self.lambda_cs = zero_coeffs
+            else:
+                self.lambda_cs = ABCDEF[2]
+            if len_ABCDEF < 4 or ABCDEF[3] is None:
+                self.lambda_ds = zero_coeffs
+            else:
+                self.lambda_ds = ABCDEF[3]
+            if len_ABCDEF < 5 or ABCDEF[4] is None:
+                self.lambda_es = zero_coeffs
+            else:
+                self.lambda_es = ABCDEF[4]
+            if len_ABCDEF < 6 or ABCDEF[5] is None:
+                self.lambda_fs = zero_coeffs
+            else:
+                self.lambda_fs = ABCDEF[5]
+        
+        # Make an array of values identifying what coefficients are zero.
+        # This may be useful for performance optimization in the future but is
+        # especially important for reducing the size of the __repr__ string.
+        self.lambda_coeffs_nonzero = lambda_coeffs_nonzero = [True]*6 if scalar else ones(6, bool)
+        for k, coeffs in enumerate([self.lambda_as, self.lambda_bs, self.lambda_cs,
+                           self.lambda_ds, self.lambda_es, self.lambda_fs]):
+            nonzero = False
+            for i in range(N):
+                r = coeffs[i]
+                for j in range(N):
+                    if r[j] != 0.0:
+                        nonzero = True
+                        break
+                if nonzero:
+                    break
+                    
+            lambda_coeffs_nonzero[k] = nonzero
 
-    model_attriubtes = ('lambda_coeffs_A', 'lambda_coeffs_B', 'lambda_coeffs_C',
-                        'lambda_coeffs_D', 'lambda_coeffs_E', 'lambda_coeffs_F')
+
+    _model_attributes = ('lambda_as', 'lambda_bs', 'lambda_cs',
+                        'lambda_ds', 'lambda_es', 'lambda_fs')
 
     def __repr__(self):
-        s = '%s(T=%s, xs=%s, ABCDEF=%s)' %(self.__class__.__name__, repr(self.T), repr(self.xs),
-                (self.lambda_coeffs_A,  self.lambda_coeffs_B, self.lambda_coeffs_C,
-                 self.lambda_coeffs_D, self.lambda_coeffs_E, self.lambda_coeffs_F))
+        
+        s = '%s(T=%s, xs=%s' %(self.__class__.__name__, repr(self.T), repr(self.xs))
+        for i, attr in enumerate(self._model_attributes):
+            if self.lambda_coeffs_nonzero[i]:
+                s += ', %s=%s' %(attr, getattr(self, attr))
+        s += ')'
         return s
 
     def to_T_xs(self, T, xs):
@@ -514,10 +847,11 @@ class Wilson(GibbsExcess):
         new.xs = xs
         new.scalar = self.scalar
         new.N = self.N
-        (new.lambda_coeffs_A, new.lambda_coeffs_B, new.lambda_coeffs_C,
-         new.lambda_coeffs_D, new.lambda_coeffs_E, new.lambda_coeffs_F) = (
-                 self.lambda_coeffs_A, self.lambda_coeffs_B, self.lambda_coeffs_C,
-                 self.lambda_coeffs_D, self.lambda_coeffs_E, self.lambda_coeffs_F)
+        (new.lambda_as, new.lambda_bs, new.lambda_cs,
+         new.lambda_ds, new.lambda_es, new.lambda_fs) = (
+                 self.lambda_as, self.lambda_bs, self.lambda_cs,
+                 self.lambda_ds, self.lambda_es, self.lambda_fs)
+        new.lambda_coeffs_nonzero = self.lambda_coeffs_nonzero
 
         if T == self.T:
             try:
@@ -569,9 +903,9 @@ class Wilson(GibbsExcess):
         else:
             lambdas = zeros((N, N))
 
-        lambdas = interaction_exp(self.T, N, self.lambda_coeffs_A, self.lambda_coeffs_B,
-                                  self.lambda_coeffs_C, self.lambda_coeffs_D,
-                                  self.lambda_coeffs_E, self.lambda_coeffs_F, lambdas)
+        lambdas = interaction_exp(self.T, N, self.lambda_as, self.lambda_bs,
+                                  self.lambda_cs, self.lambda_ds,
+                                  self.lambda_es, self.lambda_fs, lambdas)
         self._lambdas = lambdas
         return lambdas
 
@@ -600,11 +934,11 @@ class Wilson(GibbsExcess):
         except AttributeError:
             pass
 
-        B = self.lambda_coeffs_B
-        C = self.lambda_coeffs_C
-        D = self.lambda_coeffs_D
-        E = self.lambda_coeffs_E
-        F = self.lambda_coeffs_F
+        B = self.lambda_bs
+        C = self.lambda_cs
+        D = self.lambda_ds
+        E = self.lambda_es
+        F = self.lambda_fs
 
         T, N = self.T, self.N
         try:
@@ -662,10 +996,10 @@ class Wilson(GibbsExcess):
         else:
             d2lambdas_dT2 = zeros((N, N))
 
-        self._d2lambdas_dT2 = d2interaction_exp_dT2(T, N, self.lambda_coeffs_B,
-                                                                     self.lambda_coeffs_C,
-                                                                     self.lambda_coeffs_E,
-                                                                     self.lambda_coeffs_F,
+        self._d2lambdas_dT2 = d2interaction_exp_dT2(T, N, self.lambda_bs,
+                                                                     self.lambda_cs,
+                                                                     self.lambda_es,
+                                                                     self.lambda_fs,
                                                                      lambdas, dlambdas_dT, d2lambdas_dT2)
         return d2lambdas_dT2
 
@@ -700,10 +1034,10 @@ class Wilson(GibbsExcess):
             pass
 
         T, N = self.T, self.N
-        lambda_coeffs_B = self.lambda_coeffs_B
-        lambda_coeffs_C = self.lambda_coeffs_C
-        lambda_coeffs_E = self.lambda_coeffs_E
-        lambda_coeffs_F = self.lambda_coeffs_F
+        lambda_bs = self.lambda_bs
+        lambda_cs = self.lambda_cs
+        lambda_es = self.lambda_es
+        lambda_fs = self.lambda_fs
 
         try:
             lambdas = self._lambdas
@@ -719,8 +1053,8 @@ class Wilson(GibbsExcess):
         else:
             d3lambdas_dT3s = zeros((N, N))
 
-        self._d3lambdas_dT3 = d3interaction_exp_dT3(T, N, lambda_coeffs_B, lambda_coeffs_C, lambda_coeffs_E,
-                                                    lambda_coeffs_F, lambdas, dlambdas_dT, d3lambdas_dT3s)
+        self._d3lambdas_dT3 = d3interaction_exp_dT3(T, N, lambda_bs, lambda_cs, lambda_es,
+                                                    lambda_fs, lambdas, dlambdas_dT, d3lambdas_dT3s)
         return d3lambdas_dT3s
 
     def xj_Lambda_ijs(self):
@@ -1257,6 +1591,73 @@ class Wilson(GibbsExcess):
         self._gammas = gammas
         return gammas
 
+    @classmethod
+    def regress_binary_parameters(cls, gammas, xs, use_numba=False,
+                                  do_statistics=True, **kwargs):
+        # Load the functions either locally or with numba
+        if use_numba:
+            from thermo.numba import wilson_gammas_binaries as work_func, wilson_gammas_binaries_jac as jac_func
+        else:
+            work_func = wilson_gammas_binaries
+            jac_func = wilson_gammas_binaries_jac
+        
+        # Allocate all working memory
+        pts = len(xs)
+        pts2 = pts*2
+        gammas_iter, jac_iter = zeros(pts2), zeros((pts2, 2))
+
+        # Plain objective functions
+        def fitting_func(xs, lambda12, lambda21):
+            return work_func(xs, lambda12, lambda21, gammas_iter)
+            
+        def analytical_jac(xs, lambda12, lambda21):
+            return jac_func(xs, lambda12, lambda21, jac_iter)
+        
+        # The extend calls has been tested to be the fastest compared to numpy and list comprehension
+        xs_working = []
+        for xsi in xs:
+            xs_working.extend(xsi)
+        gammas_working = []
+        for gammasi in gammas:
+            gammas_working.extend(gammasi)
+            
+        xs_working = array(xs_working)
+        gammas_working = array(gammas_working)
+        
+        # Objective functions for leastsq maximum speed
+        def func_wrapped_for_leastsq(params):
+            return work_func(xs_working, params[0], params[1], gammas_iter) - gammas_working
+
+        def jac_wrapped_for_leastsq(params):
+            return jac_func(xs_working, params[0], params[1], jac_iter)
+
+        
+        fit_parameters = ['lambda12', 'lambda21']
+        return GibbsExcess._regress_binary_parameters(gammas_working, xs_working, fitting_func=fitting_func,
+                                                      fit_parameters=fit_parameters,
+                                                      use_fit_parameters=fit_parameters,
+                                                      initial_guesses=cls._gamma_parameter_guesses,
+                                                      analytical_jac=analytical_jac,
+                                                      use_numba=use_numba,
+                                                      do_statistics=do_statistics,
+                                                      func_wrapped_for_leastsq=func_wrapped_for_leastsq,
+                                                      jac_wrapped_for_leastsq=jac_wrapped_for_leastsq,
+                                                      **kwargs)
+
+    
+    # Larger value on the right always
+    _gamma_parameter_guesses = [{'lambda12': 1, 'lambda21': 1},
+                               {'lambda12': 2.2, 'lambda21': 3.0},
+                               {'lambda12': 0.015, 'lambda21': 37.0},
+                               {'lambda12': 0.5, 'lambda21': 40.0},                               
+                               {'lambda12': 1e-7, 'lambda21': .5},
+                               {'lambda12': 1e-12, 'lambda21': 1.9},
+                               {'lambda12': 1e-12, 'lambda21': 10.0},
+                               ]
+    for i in range(len(_gamma_parameter_guesses)):
+        r = _gamma_parameter_guesses[i]
+        _gamma_parameter_guesses.append({'lambda12': r['lambda21'], 'lambda21': r['lambda12']})
+    del i, r
 
 def Wilson_gammas(xs, params):
     r'''Calculates the activity coefficients of each species in a mixture
@@ -1314,10 +1715,12 @@ def Wilson_gammas(xs, params):
 
     Examples
     --------
-    Ethanol-water example, at 343.15 K and 1 MPa:
+    Ethanol-water example, at 343.15 K and 1 MPa, from [2]_ also posted online
+    http://chemthermo.ddbst.com/Problems_Solutions/Mathcad_Files/P05.01a%20VLE%20Behavior%20of%20Ethanol%20-%20Water%20Using%20Wilson.xps
+    :
 
     >>> Wilson_gammas([0.252, 0.748], [[1, 0.154], [0.888, 1]])
-    [1.8814926087178843, 1.1655774931125487]
+    [1.881492608717, 1.165577493112]
 
     References
     ----------
